
PAGE  59,132

;**************************************************************************
;**					                                 **
;**				TERM71	                                 **
;**					                                 **
;**      Created:   1-Jan-86		                                 **
;**      Passes:    5          Analysis	Options on: none                 **
;**					                                 **
;** Term71 is a utility for using the HP Portable Plus as a terminal     **
;** for the HP-71B. It can also be used with the HP-41C and HP-75	 **
;** It emulates a HP compatible HP-IL display device and recognizes the  **
;** HP escape codes. It also provides a remote keyboard feature.	 **
;** When Term71 is running, the Portable Plus is recognized as a HP-IL   **
;** device with AID=5, ID="HP45711A"					 **
;**									 **
;** Written by J-F Garnier, 1985-1986					 ** 
;** source rebuilt on Dec 2004, and given to the public domain		 **
;**************************************************************************


seg_a		segment	byte public
		assume	cs:seg_a, ds:seg_a


		org	100h

IL		equ	021h	; 1LB3 base address
ILSR		equ	IL	; status register
ILIR		equ	IL+2	; interrupt register
ILDR		equ	IL+4	; data register
ILPP		equ	IL+6	; parallel poll register
ILLA		equ	IL+8	; loop address register


term71		proc	far

start:
		mov	ax,cs
		mov	ds,ax
		call	init_dsp
		call	init_il
mainloop:
		call	do_il
		call	do_kbd
		jmp	short mainloop

term71		endp

;**************************************************************************
;                              SUBROUTINE
; do_kbd
; test for a key available
; if yes, and if HP-IL rdy, send a SRQ (Service Request) on HP-IL
;**************************************************************************

do_kbd		proc	near
		mov	ah,0Bh
		int	21h			; DOS Services  ah=function 0Bh
						;  if keybd char available,al=FF
		cmp	al,0
		je	do_kbd0			; no keys
		test	byte ptr flags,8
		jnz	do_kbd0			; already one key
		mov	ah,7
		int	21h			; DOS Services  ah=function 07h
						;  get keybd char al, no echo
		mov	dbyte,al                ; save the key
		or	byte ptr flags,8        ; set one byte ready
		in	al,ILSR
		and	al,0F0h	
		or	al,8			; set SSRQ
		out	ILSR,al			;

do_kbd0:
		retn
do_kbd		endp


;**************************************************************************
;                              SUBROUTINE
; init_il
; initialization of HP-IL stuff
;**************************************************************************

init_il		proc	near
		; wake up the 1LB3 chip
		in	al,ILSR
		mov	cx,800h
init_dly:	loop	init_dly		; Loop if cx > 0

		mov	al,1
		out	ILSR,al			; Master Clear
		dec	al			; al=0
		out	ILIR,al			; disable inttr
		dec	al			; al=0ffh
		out	ILLA,al			; no address
		inc	al			; al=0
		out	ILPP,al			; PPOLL=0
		out	ILSR,al			; SC=CA=TA=LA=0
		retn
init_il		endp


;**************************************************************************
;                              SUBROUTINE
; do_il
; manage the HP-IL protocol
;**************************************************************************

do_il		proc	near
		mov	ch,flags
		in	al,ILIR			; read Interrupt Register
		test	al,10h			; IFCR?
		jnz	ifc			;
		test	al,2			; FRNS?
		jnz	frns			; 
		test	al,4			; FRAV?
		jnz	frav			;
		test	ch,2			; active talker state?
		jz	do_il_rtn
		test	al,1			; yes, ORAV?
		jnz	sendb			; yes, send byte
do_il_rtn:	retn

ifc:		mov	ch,0                    ; reset all status flags
		in	al,ILSR
		and	al,8			; reset all but SSRQ
		or	al,6			; set CLIFCR and SLRDY
		out	ILSR,al
		jmp	do_il_end

frns:		in	al,ILDR			; Data Register
		call	ete                     ; send ETE
		jmp	do_il_end

sendb:		jmp	sendbyte

frav:		test	al,80h
		jz	doe
		test	al,20h
		jz	cmd
		jmp	rdy

doe:		in	al,ILDR			; read byte
		out	ILDR,al			; re-transmit
		test	ch,1			; talker state?
		jnz	doe0			; yes, end.
		mov	dl,al			; no, display it:
		mov	ah,2
		int	21h			; DOS Services  ah=function 02h
						;  display char dl

doe0:		retn

cmd:		in	al,ILDR			; read cmd
		mov	bl,al
		and	bl,1Fh			; bl = cmd and 31
		mov	cl,4
		shr	al,cl
		and	al,0Eh
		xor	ah,ah
		mov	si,ax
		call	word ptr tabcmd[si]	; exec cmd
		in	al,ILSR
		or	al,4			; SLRDY
		out	ILSR,al
		jmp	do_il_end

tabcmd		dw	cmd0, lad, tad, sad, cmd4, ddl, ddt, cmd7

;**** unsupported commands **************************************
sad:
ddl:
ddt:
cmd7:		retn

;**** cmd group 0 **************************************
cmd0:
		cmp	bl,4			; Selected Device Clear ?
		je	clrdev
		cmp	bl,14h			; Device Clear ?
		jne	cmd00
clrdev:		call	init_dsp
cmd00:		retn

; **** lad: listen address **************************************
lad:		cmp	bl,31			; UNL ?
		jne	lad0			; no
		test	ch,1			; yes. talker state?
		jnz	fad			; yes, ignore UNL.
		jmp	short idle		; no, go to idle.
		db	90h

lad0:		in	al,ILLA			; loop address
		cmp	al,bl			; MLA ?
		jne	fad			; no, unchanged.
		and	ch,0FCh			; yes, reset talker state
		jmp	short lstn		; and go listen
		db	90h

;**** tad: talk address **************************************
tad:		in	al,ILLA
		cmp	al,bl			; MTA?
		je	tad1			; yes
		test	ch,1			; no. talker state?
		jz	fad			; no, unchanged.
		and	ch,0FCh			; yes, reset talker state
		jmp	short idle		; and go idle
		db	90h

tad1:		and	ch,0FDh
		or	ch,1			; set talker state

lstn:		in	al,ILSR			; IL status
		and	al,8			; reset all but SSRQ
		or	al,10h			; set LA
		jmp	short fad1
		db	90h

idle:		in	al,ILSR
		and	al,8			; reset LA and TA
fad1:		out	ILSR,al			; update IL status
fad:		retn

;**** cmd group 4 **************************************
cmd4:		cmp	bl,1Ah			; AAU?
		jne	cmd40			; no
		mov	al,0FFh
		out	ILLA,al			; no address
cmd40:		retn


;**** rdy group **************************************
rdy:		in	al,ILDR			; read byte
		mov	ah,al
		test	al,80h			; start of transmission?
		jz	sot 			; yes
		cmp	al,9Fh			; no. is it an AAD?
		jae	frdy			; no, ignore.
		in	al,ILLA			; 
		test	al,80h			; already an address?
		jz	frdy			; yes, ignore.
		mov	al,ah			; no, take this address
		and	al,1Fh
		out	ILLA,al			; 
		inc	ah			; and increment AAD
frdy:		mov	al,ah
		out	ILDR,al			; re-transmit frame
do_il_end:	mov	flags,ch
		retn

sot:		test	ch,1			; talker state?
		jz	frdy			; no, end.
		cmp	al,42h			; NRD?
		jne	sda			;
		call	nrd			; yes, manage NRD
		jmp	short do_il_end

sda:		cmp	al,60h			; SDA?
		jne	sst			; 
		in	al,ILSR			; 
		and	al,8
		or	al,20h			; set TA
		out	ILSR,al			;
		or	ch,2			; talker active
		xor	al,al			; 
		out	ILIR,al			; set control bits for data byte

sendbyte:	test	ch,8			; data to send?
		jnz	sendbyt1		; yes
		in	al,ILSR
		and	al,0F0h			; no, reset TA
		out	ILSR,al
		call	eto			; and send ETO
		jmp	short do_il_end
sendbyt1:	and	ch,0F7h			; reset key available
		mov	al,dbyte		; get keycode
		out	ILDR,al			; send it
		mov	lastbyte,al		; save last sent byte
		jmp	short do_il_end

sst:		cmp	al,61h			; SST?
		jne	sai
		mov	ah,status
		and	byte ptr status,0BFh	; reset bit 6
		jmp	short sai1
		db	90h

sai:		cmp	al,63h			; SAI?
		jne	sdi
		mov	ah,5                    ; AID=5
sai1:		call	tlker			; sub_10
		mov	al,ah
		out	ILDR,al			; send frame
		mov	lastbyte,al

send_eto:	mov	al,0A0h
		out	ILIR,al			; prepare RDY frame
		mov	al,40h			; ETO
		call	senda
		call	endtalk
		jmp	short do_il_end

sdi:		cmp	al,62h			; SAI?
		jne	frdy
		call	tlker
		mov	si,offset did
sdi1:		mov	al,[si]
		inc	si
		cmp	al,0
		je	send_eto
		call	senda
		jnc	sdi1
		jmp	do_il_end

do_il		endp

did		db	'HP45711A', 0Dh, 0Ah, 0

;**************************************************************************
;                              SUBROUTINE
;**************************************************************************

tlker		proc	near
		in	al,ILSR
		and	al,8			; reset all but SSRQ
		or	al,20h			; set TA
		out	ILSR,al
		mov	al,0
		out	ILIR,al			; prepare DOE frame
		retn
tlker		endp


;**************************************************************************
;                              SUBROUTINE
;**************************************************************************

senda		proc	near
		mov	ah,al
senda1:		in	al,ILIR
		test	al,10h			; IFCR?
		jnz	sifc   
		test	al,4			; FRAV?
		jnz	sfrav
		test	al,2			; FRNS?
		jnz	sfrns
		test	al,1			; ORAV?
		jz	senda1
		mov	al,ah
		out	ILDR,al			; send frame
		mov	lastbyte,al
		clc
		retn

sifc:		pop	ax
		jmp	ifc

sfrav:		mov	dl,al
		in	al,ILDR			; read data
		mov	bl,al
		and	dl,0E0h
		cmp	dl,80h
		jne	snrd
		pop	ax
		call	endtalk
		jmp	cmd

snrd:		mov	al,bl
		cmp	al,42h			; is it NRD?
		jne	ete 			; no, ETE

;**** External Entry into Subroutine **************************************

nrd:		out	25h,al			; retransmit NRD
nrd1:		in	al,ILIR
		test	al,17h
		jz	nrd1			; wait for a frame
		and	al,80h			; data byte?
		jnz	ete   			; no, ETE
		in	al,ILDR
		cmp	al,lastbyte		; = last byte?
		jne	ete			; no, ETE

;**** External Entry into Subroutine **************************************

eto:		mov	ah,40h			; ETO
		jmp	short eot
		db	90h

sfrns:		nop

;**** External Entry into Subroutine **************************************

ete:		in	al,ILDR			; read ILDR to reset bits
		mov	ah,41h			; ETE
eot:
		mov	al,0A0h			; control bits for rdy
		out	ILIR,al
		mov	al,ah
		out	ILDR,al

senda		endp


;**************************************************************************
;                              SUBROUTINE
;**************************************************************************

endtalk		proc	near
		call	lstn                    ; listen state
		and	ch,0FDh
		stc				; Set carry flag
		retn
endtalk		endp


;**************************************************************************
;                              SUBROUTINE
; init_dsp
; initialization of display stuff
;**************************************************************************

init_dsp	proc	near
		mov	dx,offset init_str	; 
		mov	ah,9
		int	21h			; DOS Services  ah=function 09h
						;  display char string at ds:dx
		mov	byte ptr status,0
		mov	byte ptr flags,0
		retn
init_dsp	endp

init_str	db	27, 'H'		; home
		db	27, 'J'		; clear screen
		db	27, '&s1A'	; turn on transmit functions
		db	27, '*dK'	; turn on the block cursor
		db	'$'		; end of string


; *************************************************************************
; data locations
; *************************************************************************
status		db	0		; HPIL status (bit6: SRQ)
flags		db	0		; state flags (0:addr talk, 1: active talk, 2:?; 3:byte available)
lastbyte	db	0		; last sent byte
dbyte		db	0		; data byte to transmit


seg_a		ends



		end	start
